"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const clify_1 = require("@corteks/clify");
const fs_extra_1 = require("fs-extra");
const analyser_1 = require("./analyser");
const table_1 = require("./table");
const database_connector_builder_1 = require("./database/database-connector-builder");
let Main = class Main extends clify_1.CliMainClass {
    constructor() {
        super(...arguments);
        this.database = undefined;
        this.host = '127.0.0.1:3306';
        this.user = 'root';
        this.password = 'root';
        this.analyse = false;
        this.reset = false;
    }
    main() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.database)
                throw new Error('Please provide a valid database name');
            const [host, port] = this.host.split(':');
            const dbConnectorBuilder = new database_connector_builder_1.DatabaseConnectorBuilder(database_connector_builder_1.databaseEngine.MariaDB);
            const dbConnector = dbConnectorBuilder
                .setHost(host)
                .setPort(parseInt(port, 10))
                .setDatabase(this.database)
                .setCredentials(this.user, this.password)
                .build();
            try {
                if (this.analyse) {
                    let customSchema = analyser_1.dummyCustomSchema;
                    try {
                        customSchema = fs_extra_1.readJSONSync('./custom_schema.json');
                    }
                    catch (ex) {
                        console.warn('Unable to read ./custom_schema.json, this will not take any customization into account.');
                    }
                    const analyser = new analyser_1.Analyser(dbConnector, customSchema);
                    const json = yield analyser.analyse();
                    fs_extra_1.writeJSONSync('./schema.json', json, { spaces: 4 });
                    return 0;
                }
                ;
                let schema = fs_extra_1.readJSONSync('./schema.json');
                const tableService = new table_1.TableService(dbConnector, schema.maxCharLength || 255, schema.values);
                /** @todo: Remove deprecated warning */
                let useDeprecatedLines = false;
                for (const table of schema.tables) {
                    if (table.lines) {
                        useDeprecatedLines = true;
                        table.maxLines = table.lines;
                    }
                    if (table.maxLines || table.addLines) {
                        yield tableService.fill(table, this.reset);
                    }
                }
                if (useDeprecatedLines)
                    console.warn('DEPRECATED: Table.lines is deprecated, please use table.maxLines instead.');
                /****************/
            }
            catch (ex) {
                if (ex.code == 'ENOENT') {
                    console.error('Unable to read from schema.json. Please run with --analyse first.');
                }
                else {
                    console.error(ex);
                }
            }
            finally {
                console.log('Close database connection');
                yield dbConnector.destroy();
            }
            return 0;
        });
    }
};
__decorate([
    clify_1.CliParameter({ alias: 'db', demandOption: true, description: 'database', }),
    __metadata("design:type", Object)
], Main.prototype, "database", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", String)
], Main.prototype, "host", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", String)
], Main.prototype, "user", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", String)
], Main.prototype, "password", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "analyse", void 0);
__decorate([
    clify_1.CliParameter(),
    __metadata("design:type", Boolean)
], Main.prototype, "reset", void 0);
Main = __decorate([
    clify_1.CliMain
], Main);
